# Documentation I have handy for mol files is from October 1996.

from Martel import Re, Rep, Group, Str, Opt, RepN, Integer, ToEol
from Martel import Rep1, Alt, AnyEol


def Count(name, count):
    return Group(name, Re("." * count))

# Allow the count to be truncated by the end of line
def EndCount(name, count):
    return Group(name, Re("[^\R]" * count + \
                          r"|(?=.{0,%d}\R).+" % (count-1)))

# aaabbblllfffcccsssxxxrrrpppiiimmmvvvvvv
# aaa = num_atoms
#    bbb = num_bonds
#       lll = num_atom_lists
#          fff = obsolete
#             ccc = chiral_flag
#                sss = num_stext_entries
#                   xxx = num_reaction_components
#                      rrr = num_reactants
#                         ppp = num_products
#                            iii = num_intermediates
#                               mmm = num_properties (should be 999)
#                                  vvvvvv = ctab_version

counts_line = Count("num_atoms", 3) + \
              Count("num_bonds", 3) + \
              Count("num_atom_lists", 3) + \
              Count("obsolete", 3) + \
              Count("chiral_flag", 3) + \
              Count("num_stext_entries", 3) + \
              Count("num_reaction_components", 3) + \
              Count("num_reactants", 3) + \
              Count("num_products", 3) + \
              Count("num_intermediates", 3) + \
              Count("num_properties", 3) + \
              Count("ctab_version", 6) + \
              AnyEol()

# xxxxx.xxxxyyyyy.yyyyzzzzz.zzzz aaaddccssshhhbbbvvvHHHrrriiimmmnneee
# xxxxx.xxxx = x (coordinate)
#           yyyyy.yyyy = y (coordinate)
#                     zzzzz.zzzz = z (coordinate)
#                                aaa = atom_symbol
#                                   dd = mass_difference
#                                     cc = charge
#                                       sss = atom_stereo_parity
#                                          hhh = hydrogen_count
#                                             bbb = stereo_care_box
#                                                vvv = valence
#                                                   HHH = HO_designator
#                                                      rrr = reaction_type
#                                       reaction_number = iii
#                                          mapping_number =  mmm
#                                               inv_ret_flag =  nn
#                                             exact_change_flag = eee

atom_line = Count("x", 10) + \
            Count("y", 10) + \
            Count("z", 10) + \
            Str(" ") + \
            EndCount("atom_symbol", 3) + \
            Opt(Count("mass_difference", 2) + \
                Count("charge", 3) + \
                Opt(Count("atom_stereo_parity", 3) + \
                    Count("hydrogen_count", 3) + \
                    Count("stereo_care_box", 3) + \
                    Count("valence", 3) + \
                    Opt(Count("HO_designator", 3) + \
                        Count("reaction_type", 3) + \
                        Count("reaction_number", 3) + \
                        Count("mapping_number", 3) + \
                        Count("inv_ret_flag", 3) + \
                        Count("exact_change_flag", 3))
                    )
                ) + \
            AnyEol()

#atom_block = Group("atom_block", Rep(atom_line))
atom_block = Group("atom_block", RepN(atom_line, "num_atoms"))


# 111222tttsssxxxrrrccc
# 111 = atom_id1
#    222 = atom_id2
#       ttt = bond_type
#          sss = bond_stereo
#             xxx = ignore
#                rrr = bond_topology
#                   ccc = center_status
bond_line = Count("atom_id1", 3) + \
            Count("atom_id2", 3) + \
            Count("bond_type", 3) + \
            Opt( Count("bond_stereo", 3) + \
                 Opt(Count("ignore", 3) + \
                     Count("bond_topology", 3) + \
                     Opt(Count("center_status", 3))
                     )
                 ) + \
            AnyEol()

bond_block = Group("bond_block", RepN(bond_line, "num_bonds"))

# aaa kSSSSn 111 222 333 444 555
atom_list = Count("atom_id", 3) + \
            Str(" ") + \
            Count("is_not_normal", 1) + \
            Str("    ") + \
            Count("num_entries", 1) + \
            Re("( (?P<atomic_number>...)){1,5}") + \
            AnyEol()

atom_list_block = RepN(atom_list, "num_atom_lists")

# xxxxx.xxxxyyyyy.yyyy
# TTTT....

#stext = Re(r"(?!\w)(?P<stext_x>..........)(?P<stext_y>[^\R]*)\R") + \
#        Re(r"(?P<stext_text>[^\R]*)\R")
stext = Re(r"(?P<stext_y>[^\R]*)\R") + \
        Re(r"(?P<stext_text>[^\R]*)\R")

stext_block = RepN(stext, "num_stext_entries")

# Properties

# Untested
atom_alias = Group("atom_alias",
                   Re(r"A  (?P<atom_number>...)\R(?P<alias_text>[^\R]*)\R"))

# Untested
atom_value = Group("atom_value",
                   Re(r"V  (?P<atom_number>...)\R(?P<value_text>[^\R]*)\R"))

# Barely tested
atom_group = Group("group_abbreviation",
                   Re(r"G  (?P<atom_number1>...)(?P<atom_number2>...)\R" \
                      r"(?P<abbreviation_label>[^\R]*)\R"))

# Untested
skip = Group("skip_block",
             Re(r"S  SKP(?P<count>...)\R([^\R]*\R){count}"))

def make_id_value(line_name, ident_text):
    return Group(line_name,
                 Re(r"M  " + ident_text + "(?P<count>...)" + \
                    r"( (?P<atom_id>...) (?P<value>...)){count}\R"))

# THIS HAS BEEN TESTED!
charge = make_id_value("charge", "CHG")

# THIS HAS BEEN TESTED!
radical = make_id_value("radical", "RAD")

# THIS HAS BEEN TESTED!
isotope = make_id_value("isotope", "ISO")

# Untested
ring_bond_count = make_id_value("ring_bond_count", "RBD")

# Untested
substitution_count = make_id_value("substitution_count", "SUB")

# Untested
unsaturated_atom = make_id_value("unsaturated_atom", "UNS")

# Untested, and most likely wrong
link_atom = Group("link_atom",
                  Re(r"M  LIN(?P<count>...) (?P<link_atom>...)" \
                     r" (?P<substituent_count>...)" \
                     r" (?P<substituent_atom1>...)" \
                     r" (?P<substituent_atom2>...)" \
                     r"( (?P<substituent_atom>...)){count}\R"))

# THIS HAS BEEN TESTED!
# 'M_atom_list' to distingush it from the old-style atom_list
M_atom_list = Group("atom_list",
                    Re(r"M  ALS (?P<atom_number>...)(?P<count>...) " \
                       r"(?P<exclusion>[TF]) " \
                       r"(?P<entry>.(?=\R)|..(?=\R)|...(?=\R)|....){count}\R"))

######################

# All sorts of untested elements because I don't have data files for
# them.  I copied some files from the documentation, but that's not
# enough to know if they are real.

# Untested
attachment_point = make_id_value("attachment_point", "APO")

# Untested
rgroup_label_location = make_id_value("rgroup_label_location", "RGP")

# Untested
rgroup_logic = Group("rgroup_logic",
                     Re(r"M  LOG(?P<count>...) (?P<rgroup_number>...) " \
                        r"(?P<other_rgroup>...) (?P<restH_property>...) " \
                        r"(?P<rgroup_range>...)\R"))

# Barely tested
sgroup_type = make_id_value("sgroup_type", "STY")

# Untested
sgroup_subtype = make_id_value("sgroup_subtype", "SST")

# Barely tested
sgroup_labels = make_id_value("sgroup_labels", "SLB")

# Untested
sgroup_connectivity = make_id_value("sgroup_connectivity", "SCN")

# Untested
sgroup_expansion = Group("sgroup_expansion",
                         Re(r"M  SDS EXP(?P<count>...)" \
                            r"( (?P<sgroup_index>...)){count}\R"))

# Barely tested
sgroup_atom_list = Group("sgroup_atom_list",
                         Str("M  SAL ") + \
                         Re(r"(?P<sgroup_index>...)(?P<count>...)" \
                            r"( (?P<atom_id>...){count})\R"))

# Untested
sgroup_bond_list = Group("sgroup_bond_list",
                         Str("M  SBL ") + \
                         Re(r"(?P<sgroup_index>...)(?P<count>...)" \
                            r"( (?P<bond_id>...){count})\R"))

# Untested
multiple_group_parent_atom_list = Group("multiple_group_parent_atom_list",
                         Str("M  SPA ") + \
                         Re(r"(?P<sgroup_index>...)(?P<count>...)" \
                            r"( (?P<atom_id>...){count})\R"))

# Barely tested
sgroup_subscript = Group("sgroup_subscript",
                         Str("M  SMT ") + \
                         Re("(?P<sgroup_index>...) ") + \
                         ToEol("text"))

# Untested, and most likely wrong
sgroup_correspondence = Group("sgroup_correspondence",
                              Str("M  CRS ") + \
                              Count("sgroup_index", 3) + \
                              Count("count", 3) + \
                              Re("( (?P<bond1>...) (?P<bond2>...) " \
                                 "(?P<bond3>...)){count}") + \
                              AnyEol())
                              

# Untested, na
sgroup_display_information = Group("sgroup_display_information",
                                   Str("M  SDI ") + \
                                   Count("sgroup_index", 3) + \
                                   Count("count", 3) + \
                                   Count("x1", 10) + \
                                   Count("x2", 10) + \
                                   Count("y1", 10) + \
                                   Count("y2", 10) + \
                                   AnyEol())

# Untested
superatom_bond_and_vector_information = \
                Group("superatom_bond_and_vector_information",
                      Str("M  SBV ") + \
                      Count("sgroup_index", 3) + Str(" ") + \
                      Count("bond_id", 3) + \
                      Count("x1", 10) + \
                      Count("x2", 10) + \
                      AnyEol()
                      )

# Untested
data_sgroup_field_description = Group("data_sgroup_field_description",
                                      Str("M  SDT ") + \
                                      Count("sgroup_idex", 3) + Str(" ") + \
                                      Count("field_name", 30) + \
                                      Count("field_type", 2) + \
                                      Count("field_units", 20) + \
                                      Count("query_type", 2) + \
                                      ToEol("query_operator"))
                                      

# Untested
data_sgroup_display_information = Group("data_sgroup_display_information",
                                        Str("M  SDD ") + \
                                        Count("sgroup_index", 3) + Str(" ") + \
                                        Count("x", 10) + \
                                        Count("y", 10) + \
                                        Count("reserved1", 3) + \
                                        Count("data_dispay", 1) + \
                                        Count("placement", 1) + \
                                        Count("units", 1) + \
                                        Count("reserved2", 1) + \
                                        Count("num_chars_to_display", 3) + \
                                        Count("num_lines_to_display", 3) + \
                                        Count("reserved3", 2) + \
                                        Count("tag_character", 1) + \
                                        Count("dasp_position", 1) + \
                                        Count("reserved4", 2) + \
                                        AnyEol())

# Untested
data_sgroup_data_SCD = Group("data_sgroup_data_SCD",
                             Str("M  SCD ") + \
                             Count("sgroup_index", 3) + Str(" ") + \
                             ToEol("data"))

data_sgroup_data_SED = Group("data_sgroup_data_SED",
                             Str("M  SED ") + \
                             Count("sgroup_index", 3) + Str(" ") + \
                             ToEol("data"))

data_sgroup_data = Rep(data_sgroup_data_SCD) + data_sgroup_data_SED

# Untested
sgroup_hierarchy_information = Group("sgroup_hierarchy_information",
                                     Str("M  SPL") + \
                                     Count("count", 3) + \
                                Re(r"( (?P<child_sgroup_index>...) " \
                                   r"(?P<parent_sgroup_index>...)){count}\R"))

# Untested
sgroup_component_numbers = Group("sgroup_component_numbers",
                                 Str("M  SNC") + \
                                 Count("count", 3) + \
                                 Re(r"( (?P<component_index>...) " \
                                    r"(?P<component_index>...)){count}\R"))

# Untested
x3D_features_count_line = Group("_3D_features_count_line",
                                Str("M  $3D") + \
                                Count("_3d_count", 3) + \
                                AnyEol())

x3D_features_data_line = Group("_3D_features_data_line",
                               Str("M  $3D") + \
                               Re("  [0-9]| [0-9]{2}|[0-9]{3}"))
                               
common_identification = Count("color_number", 3) + \
                        Opt(Str(" ") + \
                            Group("feature_name", Re("[^\s]{0,32}")) + \
                            Group("comment", Re("[^\s]{0,32}"))) + \
                        AnyEol()

#-- $3D -1

x3D_feature_point_dist_head = Group("_3D_feature_point_head",
                                     Str("M  $3D -1") + common_identification)

x3D_feature_point_data = Group("_3D_feature_point_data",
                               Str("M  $3D") + \
                               Count("point1", 3) + \
                               Count("point2", 3) + \
                               Count("distance", 10) + \
                               AnyEol()
                               )
x3D_feature_point_dist = x3D_feature_point_dist_head + \
                         x3D_feature_point_data

#-- $3D -2

x3D_feature_point_frac_head = Group("_3D_feature_point_frac_head",
                                    Str("M  $3D -2") + common_identification)

x3D_feature_point_frac = x3D_feature_point_frac_head + \
                         x3D_feature_point_data


#-- $3D -3
x3D_feature_point_normal_head = Group("_3D_feature_point_normal_head",
                                      Str("M  $3D -3") + common_identification)
x3D_feature_point_normal_data = Group("_3D_feature_point_data",
                                      Str("M  $3D") + \
                                      Count("point", 3) + \
                                      Count("line", 3) + \
                                      Count("distance", 10) + \
                                      AnyEol())
x3D_feature_point_normal = x3D_feature_point_normal_head + \
                           x3D_feature_point_normal_data


#-- $3D -4
x3D_feature_best_fit_head = Group("_3D_feature_best_fit_head",
                                  Str("M  $3D -4") + common_identification)
x3D_feature_best_fit_count = Group("_3D_feature_best_fit_count",
                                   Str("M  $3D") + \
                                   Count("num_points", 3) + \
                                   Count("deviation", 10) + \
                                   AnyEol()
                                   )

# This is evil.  I need to count "num_points" times over the items,
# and there are at most 20 items per line.  So I allow the item to
# start with an option "M $3D" and end with an optional "\n".  The
# whole thing is repeated "num_points" times.  It does allow
# "M $3D 1M  $3D 2\n" as a valid match. If that's a problem, use
# lookahead assertions.  This will consume the final "\n" because
# of greed.  Hmm, I think I'm missing the .._fit_data group.
x3D_feature_best_fit_data = RepN(Opt(Str("M  $3D")) + \
                                 Count("item_id", 3) + \
                                 Opt(AnyEol()),
                                 "num_points")

x3D_feature_best_fit = x3D_feature_best_fit_head + \
                       x3D_feature_best_fit_count + \
                       x3D_feature_best_fit_data
                       
#-- $3D -5
x3D_feature_plane_points_head = Group("_3D_feature_plane_points_head",
                                      Str("M  $3D -5") + common_identification)
x3D_feature_plane_points_count = Group("_3D_feature_plain_points_count",
                                       Str("M  $3D") + \
                                       Count("num_points", 3) + \
                                       Count("deviation", 10) + \
                                       AnyEol())
# More evil
x3D_feature_plane_points_data = RepN(Opt(Str("M  $3D")) + \
                                 Count("item_id", 3) + \
                                 Opt(AnyEol()),
                                 "num_points")
x3D_feature_plane_points = x3D_feature_plane_points_head + \
                           x3D_feature_plane_points_count + \
                           x3D_feature_plane_points_data


#-- $3D -6
x3D_feature_plane_point_line = \
        Group("_3D_feature_plane_point_line_head",
              Str("M  $3D -6") + common_identification) + \
        Group("_3D_feature_plane_point_line_data",
              Str("M  $3D") + \
              Count("point_id", 3) + \
              Count("line_id", 3) + \
              AnyEol())


######################

properties = atom_group | charge | radical | isotope | M_atom_list | \
             sgroup_type | sgroup_labels | sgroup_atom_list | sgroup_subscript

ctab_block = counts_line + atom_block + bond_block + atom_list_block + \
             stext_block + Rep(properties)

header_block = ToEol("molecule_name") + \
               ToEol("creation_info") + \
               ToEol("comment")

# IIPPPPPPPPMMDDYYHHmmddSSssssssssssEEEEEEEEEEEERRRRRR
# (no one really uses this, do they?  Provided just in case, and
# hasn't been tested.)
creation_info_format = Count("initials", 2) + \
                       Count("program_name", 8) + \
                       Count("month", 2) + \
                       Count("day", 2) + \
                       Count("year", 2) + \
                       Count("hour", 2) + \
                       Count("minute", 2) + \
                       Count("dimensional_codes", 2) + \
                       Count("scaling_factor1", 2) + \
                       EndCount("scaling_factor2", 10) + \
                       Opt(EndCount("energy", 12) + \
                           Opt(Count("registry_number", 6))
                           )




end = Re(r"M  END\R")

molfile_format = Group("molfile", header_block + ctab_block + end)
#molfile_format = header_block + ctab_block + end

########

# Rxnfiles
rxn_line1 = Str("$RXN") + AnyEol()
rxn_line2 = AnyEol()
rxn_line3 = (Re(r"(?P<program>.{15})" \
                r"(?P<date>(?P<month>..)(?P<day>..)(?P<year>....)" \
                            r"(?P<hour>..)(?P<minute>..))") + \
             Opt(Group("registry_number", Re(".{8}"))) + \
             AnyEol()) | \
             AnyEol()  # a blank line can bs substutued for the data

rxn_line4 = ToEol("comment")
rxn_line5 = Group("num_rxn_reactants", Re(r"...")) + \
            Group("num_rxn_products", Re(r"...")) + \
            AnyEol()

rxn_header = rxn_line1 + \
             rxn_line2 + \
             rxn_line3 + \
             rxn_line4 + \
             rxn_line5
             
rxn_reactants = Group("reactants",
                      Str("$MOL") + AnyEol() + \
                      RepN(molfile_format, "num_rxn_reactants"))

rxn_products = Group("products",
                     Str("$MOL") + AnyEol() + \
                     RepN(molfile_format, "num_rxn_products"))

rxnfile_format = Group("rxnfile",
                       rxn_header + \
                       rxn_reactants + \
                       rxn_products)


# RDFile-specific data

# Line 1: '$RDFILE 1'  # But some files have a space after the number!
# Line 2: '$DATM    12/21/99 13:15'
# I've also seen
# Line 2: '$DATM 1/21/99 13:15:25'
# and even
# Line 2: '$DATM 1/2/99 3:15:2'
# Maybe I shouldn't even attempt to parse the line.  The format spec
# says it's ignored.
rdfile_header = Group("header",
                      Str("$RDFILE 1") + Re(" *") + AnyEol() + \
                      Str("$DATM") + Re(" +") + \
    Re(r"(?P<date>(?P<month>\d\d?)/(?P<day>\d\d?)/(?P<year>\d\d(\d\d)?)) " \
       r"(?P<time>(?P<hour>\d\d?):(?P<minute>\d\d?)(:(?P<second>\d\d?))?)\R")
                      )
mireg = Str("$MIREG ") + Group("internal_regno", Re(r"[^$\R]+"))
mereg = Str("$MEREG ") + Group("external_regno", Re(r"[^$\R]+"))

# The MFMT line can be blank padded!
rdfile_midentifier1 = Group("molecule_identifier",
                            Str("$MFMT") + \
                            Opt( Str(" ") + mireg + \
                                 Opt(Str(" ") + mereg)) + \
                            ToEol())
rdfile_midentifier2 = Group("molecule_identifier", mireg + AnyEol())
rdfile_midentifier3 = Group("molecule_identifier", mereg + AnyEol())

rireg = Str("$RIREG ") + Integer("internal_regno")
rereg = Str("$REREG ") + Integer("external_regno")
rdfile_ridentifier1 = Group("reaction_identifier",
                            Str("$RFMT") + \
                            Opt(Str(" ") + rireg + \
                                Opt(Str(" ") + rereg)) + \
                            AnyEol())

# Don't have documentation for the CPSS-style rxnfile
rdfile_ridentifier2 = Group("reaction_identifier",
                            Str("$PCRXN") + \
                            Opt(Str(" ") + rireg + \
                                Opt(Str(" ") + rereg)) + \
                            AnyEol())
rdfile_ridentifier3 = Group("reaction_identifier", rireg + AnyEol())
rdfile_ridentifier4 = Group("reaction_identifier", rereg + AnyEol())

rdfile_identifier = (rdfile_midentifier1 + molfile_format) | \
                     rdfile_midentifier2 | \
                     rdfile_midentifier3 | \
                    (rdfile_ridentifier1 + rxnfile_format) | \
                     rdfile_ridentifier3 | \
                     rdfile_ridentifier4

                     

dtype = Str("$DTYPE ") + ToEol("field_name")
# datums can fold over several lines.  The escape mechanism is not defined.
# It appears to be a '+' at the 81st column
datum = Group("datum_block",
              Str("$DATUM ") + (Re(r"(?P<datum>.{0,73})\R") | \
                                (Re(r"(?P<datum>.{73})\+\R") + \
                                 Rep(Re(r"(?P<datum>.{80})\+\R")) + \
                                 ToEol("datum"))
                                )
              )
data_block = Group("data_block",
                   Rep(dtype + datum))
data_block = Rep(dtype + datum)
                   

rdfile_format = rdfile_header + Rep1(Group("rdfile_record",
                                           rdfile_identifier + \
                                           data_block))

########

field_name = Str("<") + Group("field_name", Re(r"[^>\R]+")) + Str(">")
external_registry_number = Str("(") + \
           Group("external_registry_number", Re(r"[^\R)]+")) + Str(")")


item_header = Str(">  ") + \
              Alt(field_name + Opt(Str(" ") + external_registry_number),
                  Group("database_field", Re(r"[^<\R][^\R]*"))) + \
              AnyEol()


data_element = Group("data_element",
                     item_header + \
                     Rep(Group("data_value", Re(r"[^\R]+\R"))) + \
                     AnyEol())

sdfile_end = Str("$$$$") + AnyEol()

sdfile_record = Group("sdfile_record",
                      molfile_format + \
                      Group("data_element_block",
                            Rep(data_element)) + \
                      sdfile_end)
sdfile_record = molfile_format + \
                Group("data_element_block",
                      Rep(data_element)) + \
                      sdfile_end

sdfile_format = Group("sdfile", Rep(sdfile_record))
sdfile_format = Rep(sdfile_record)

########

format = molfile_format
