"""Read the Block 12.0 format"""

import Martel
from Martel import RecordReader

def UpTo(c, name):
    return Martel.Group(name, Martel.Re(r"[^%s\R]+" % c))

def Text(count, name):
    return Martel.Group(name, Martel.Re("." * count))

version = Martel.Group("version", \
             Martel.Re(r"(?P<major_version>\d+)\.(?P<minor_version>\d+)"))
date = Martel.Group("date", \
             Martel.Re(r"(?P<month>\w+) (?P<year>\d+)"))

header = Martel.Re(" *Blocks Database Version ") + version + \
         Martel.Str(", ") + date + Martel.AnyEol() + \
         Martel.Rep(Martel.Re(r"(?!ID)[^\R]*\R"))


# ID   kringle; BLOCK
ID_line = Martel.Str("ID   ") + UpTo(";", "id") + \
          Martel.Str("; BLOCK") + Martel.AnyEol()

# AC   IPB000001A; distance from previous block=(10,266)
# AC   IPB000007E; distance from previous block=(-1,26)
AC_line = Martel.Str("AC   ") + UpTo(";", "accession") + \
          Martel.Str("; distance from previous block=(") + \
          Martel.SignedInteger("block_start") + Martel.Str(",") + \
          Martel.Integer("block_end") + Martel.Str(")") + Martel.AnyEol()

# DE   Protein phosphatase 2A regulatory subunit PR55
DE_line = Martel.Str("DE   ") + Martel.ToEol("description")

# BL   ASL;  width=47; seqs=20; 99.5%=1771; strength=1740
# BL   QGV;  width=14; seqs=104; 99.5%=824; strength=1070
BL_data = Martel.Str("width=") + Martel.Integer("width") + Martel.Str(";") + \
          Martel.Str(" seqs=") + Martel.Integer("seqs") + Martel.Str(";") + \
          Martel.Str(" 99.5%=") + Martel.Integer("count") + Martel.Str(";") + \
          Martel.Str(" strength=") + Martel.Integer("strength")

BL_line = Martel.Str("BL   ") + \
          Text(3, "name") + Martel.Str(";  ") + \
          BL_data + \
          Martel.AnyEol()

# CYTI_VIGUN|Q06445  (  49) QVVSGTLYTITLEA  13
# Q41897             (  85) QVVAGTMYYLTIEV   6
# 2ABA_HUMAN|Q00007  (  26) AEADIISTVEFNHSGELLATGDKGGRVVIFQQEQENK  32
data_line = Martel.Group("sequence_id", Martel.Re(r"[^/\R][^ ]*")) + \
            Martel.Re(r" *\( *") + Martel.SignedInteger("position") + \
            Martel.Str(") ") + \
            Martel.Group("segment", Martel.Re(r"[^ ]+")) + \
            Martel.Re(" +") + \
            Martel.Integer("weight") + \
            Martel.AnyEol()

blank_line = Martel.AnyEol()

cluster = Martel.Group("cluster",
                       Martel.Rep1(data_line) + \
                       Martel.Rep(blank_line + Martel.Rep1(data_line))
                       )


end_line = Martel.Str("//") + Martel.AnyEol()

record = Martel.Group("blocks_record",
                      ID_line + \
                      AC_line + \
                      DE_line + \
                      BL_line + \
                      cluster + \
                      end_line)

format = Martel.HeaderFooter("blocks",
                             header, RecordReader.Until, ("ID",),
                             record, RecordReader.EndsWith, ("//",),
                             None, None, None,
                             )

