"""THE ENZYME NOMENCLATURE DATABASE Release 26.0, May 2000"""

# 2.3.1.61 contains an ESC character!

import Martel
from Martel import RecordReader

import swissprot38


header = Martel.Group("header", Martel.Re(r"(CC(?P<foo>[^\R]*)\R)+//\R"))
header = Martel.Re(r"CC   (?P<hyphens>-+)\R") + \
         Martel.Rep(Martel.Group("line", Martel.Re("[^\R]*\R")))

ec_number = Martel.Re(r"\d+\.\d+\.\d+\.\d+")

##  ID  Identification                         (Begins each entry; 1 per entry)
ID_line = Martel.Str("ID   ") + \
          Martel.Group("EC_number", ec_number) + \
          Martel.AnyEol()

##  DE  Description (official name)            (>=1 per entry)
DE_block = swissprot38.DE_block

##  AN  Alternate name(s)                      (>=0 per entry)
nonterminal_AN_line = Martel.Str("AN   ") + \
                      Martel.Re(r"(?P<alternate_name>([^.\R]+|(?!\.\R)\.)*)\R")
terminal_AN_line = Martel.Str("AN   ") + \
                   Martel.Re(r"(?P<alternate_name>([^.\R]+|(?!\.\R)\.)*)\.\R")

AN_subblock = Martel.Group("AN_subblock",
                           Martel.Rep(Martel.Group("nonterminal_AN_line",
                                      nonterminal_AN_line)) + \
                           Martel.Group("terminal_AN_line",
                                        terminal_AN_line))

AN_block = Martel.Group("AN_block",
                        Martel.Rep(AN_subblock))

##  CA  Catalytic activity                     (>=0 per entry)
CA_line = swissprot38.Simple("CA", "catalytic_activity")
CA_block = Martel.Group("CA_block", Martel.Rep1(CA_line))


##  CF  Cofactor(s)                            (>=0 per entry)
CF_line = swissprot38.Simple("CF", "cofactor")
CF_block = Martel.Group("CF_block", Martel.Rep1(CF_line))

##  CC  Comments                               (>=0 per entry)

# 1.1.1.249 has "CC    -!-"
# instead of    "CC   -!-"
# so does 2.8.2.17

CC_begin = Martel.Group("CC",
                        Martel.Re("CC    ?-!- (?P<comment_text>[^\R]*)\R"))
CC =       Martel.Group("CC",
                        Martel.Re("CC       (?P<comment_text>[^\R]*)\R"))

single_comment = Martel.Group("comment", 
                              CC_begin +
                              Martel.Rep(CC)
                              )


comment_block = Martel.Group("comment_block",
                             Martel.Rep1(single_comment))

##  DI  Disease(s) associated with the enzyme  (>=0 per entry)
# Most lines end in ".", one in "," and one in 0.
# Some have a space after the "MIM:
DI_line = Martel.Str("DI   ") + \
          Martel.Re(r"(?P<disease_name>[^;]+); MIM: ?(?P<mim_number>\d+)") + \
          Martel.Re("[.,]?") + \
          Martel.AnyEol()

DI_block = Martel.Group("DI_block",
                        Martel.Rep1(DI_line))

##  PR  Cross-references to PROSITE            (>=0 per entry)
PR_line = Martel.Str("PR   PROSITE; ") + \
          Martel.Group("prodoc_entry", Martel.Re("[^;\R]+")) + \
          Martel.Str(";") + \
          Martel.AnyEol()

PR_block = Martel.Rep1(PR_line)

##  DR  Cross-references to SWISS-PROT         (>=0 per entry)
swissref = Martel.Group("swissprot_accession", Martel.Re(r"[^,\R]+")) + \
          Martel.Str(", ") + \
          Martel.Group("entry_name", Martel.Re(r"[^\s;]+")) + \
          Martel.Re(" *;")

DR_line = Martel.Str("DR   ") + \
          swissref + \
          Martel.Rep(Martel.Str("  ") + swissref) + \
          Martel.AnyEol()
DR_block = Martel.Group("DR_block", Martel.Rep1(DR_line))


##  //  Termination line                       (Ends each entry; 1 per entry)
end = Martel.Str("//") + Martel.AnyEol()
                       

record = Martel.Group("enzyme26_record",
                      ID_line + \
                      DE_block + \
                      Martel.Opt(AN_block) + \
                      Martel.Opt(CA_block) + \
                      Martel.Opt(CF_block) + \
                      Martel.Opt(comment_block) + \
                      Martel.Opt(DI_block) + \
                      Martel.Opt(PR_block) + \
                      Martel.Opt(DR_block) + \
                      end
                      )

format = Martel.HeaderFooter("enzyme26",
                             header, RecordReader.EndsWith, ("//", ),
                             record, RecordReader.EndsWith, ("//", ),
                             None, None, None,
                             )
