"""Parse the file reference from the README claiming:

The taxonomy database of the International Sequence Database Collaboration
contains the names of all organisms that are represented in the sequence
databases with at least one nucleotide or protein sequence.
(For more information see http://www.ncbi.nlm.nih.gov/Taxonomy/).
 
The data base is maintained at NCBI. Daily bcp-like dump files are available
from ftp://ncbi.nlm.nih.gov/pub/taxonomy/. This directory contains the flatfile
version produced at EBI once a week for use in SRS.
"""

import Martel
from Martel import RecordReader

def key_value(name, tag):
    return Martel.Re("%s *: (?P<%s>[^\R]+)\R" % (name, tag))

acronym = key_value("ACRONYM", "acronym")
anamorph = key_value("ANAMORPH", "anamorph")
blast_name = key_value("BLAST NAME", "blast_name")
common_name = key_value("COMMON NAME", "common_name")
equivalent_name = key_value("EQUIVALENT NAME", "equivalent_name")
gc_id = key_value("GC ID", "gc_id")
id = key_value("ID", "id")
in_part = key_value("IN-PART", "in_part")
includes = key_value("INCLUDES", "includes")
mgc_id = key_value("MGC ID", "mgc_id")
misnomer = key_value("MISNOMER", "misnomer")
misspelling = key_value("MISSPELLING", "misspelling")
parent_id = key_value("PARENT ID", "parent_id")
preferred_acronym = key_value("PREFERRED ACRONYM", "preferred_acronym")
preferred_common_name = key_value("PREFERRED COMMON NAME", "common_name")
rank = key_value("RANK", "rank")
scientific_name = key_value("SCIENTIFIC NAME", "scientific_name")
synonym = key_value("SYNONYM", "synonym")
teleomorph = key_value("TELEOMORPH", "teleomorph")

end = Martel.Str("//") + Martel.AnyEol()

# Need to check the order against SRS
record = Martel.Group("taxonomy_record",
                      id + \
                      parent_id + \
                      rank + \
                      gc_id + \
                      Martel.Opt(mgc_id) + \
                      scientific_name + \
                      Martel.Opt(preferred_common_name) + \
                      Martel.Rep(synonym) + \
                      Martel.Opt(preferred_acronym) + \
                      Martel.Rep(common_name) + \
                      Martel.Rep(acronym) + \
                      Martel.Rep(anamorph) + \
                      Martel.Rep(teleomorph) + \
                      Martel.Rep(includes) + \
                      Martel.Rep(in_part) + \
                      Martel.Opt(blast_name) + \
                      Martel.Rep(misnomer) + \
                      Martel.Rep(misspelling) + \
                      Martel.Rep(equivalent_name) + \
                      end)

format = Martel.ParseRecords("taxonomy", record,
                             RecordReader.EndsWith, ("//\n", ))
