"""Parser for the TrEMBL entries, Release 14, June 2000"""

import Martel
from Martel import RecordReader
import swissprot38

##ID = Martel.Group("ID", Martel.Re(
##    r"ID   (?P<entry_name>\w+) +(?P<data_class_table>PRELIMINARY); +"
##    r"(?P<molecule_type>\w+); +(?P<sequence_length>\d+) AA\.\R"))
ID = swissprot38.ID

AC = swissprot38.AC

DT_created = Martel.Group("DT_created", Martel.Re(
    r"DT   (?P<day>\d\d)-(?P<month>...)-(?P<year>\d{4}) \(TrEMBLrel. "\
    r"(?P<release>\d\d), Created\)\R"
    ))
DT_seq_update = Martel.Group("DT_seq_update", Martel.Re(
    r"DT   (?P<day>\d\d)-(?P<month>...)-(?P<year>\d{4}) \(TrEMBLrel. "\
    r"(?P<release>\d\d), Last sequence update\)\R"
    ))
DT_ann_update = Martel.Group("DT_ann_update", Martel.Re(
    r"DT   (?P<day>\d\d)-(?P<month>...)-(?P<year>\d{4}) \(TrEMBLrel. "\
    r"(?P<release>\d\d), Last annotation update\)\R"
    ))

DE_block = swissprot38.DE_block
GN_block = swissprot38.GN_block
OS_block = swissprot38.OS_block
OG_block = swissprot38.OG_block
OC_block = swissprot38.OC_block
reference = swissprot38.reference
comment = swissprot38.comment
DR_block = swissprot38.DR_block
KW_block = swissprot38.KW_block
feature_block = swissprot38.feature_block

SQ = Martel.Group("SQ",
   Martel.Re("SQ   SEQUENCE\s+(?P<sequence_length>\d+) AA;" \
             "\s+(?P<molecular_weight>\d+) MW;" \
             "\s+(?P<crc64>\w+) CRC64;\R")
                  )
SQ_data = Martel.Group("SQ_data",
                       Martel.Str("     ") + \
                       Martel.ToEol("sequence"))

sequence = Martel.Group("sequence_block", Martel.Group("SQ_data_block",
                                                 SQ + Martel.Rep(SQ_data)))


end = swissprot38.end

record = Martel.Group("trembl14_record", \
    ID + \
    AC + \
    DT_created + \
    DT_seq_update + \
    DT_ann_update + \
    Martel.Opt(DE_block) + \
    Martel.Opt(GN_block) + \
    Martel.Opt(OS_block) + \
    Martel.Opt(OG_block) + \
    Martel.Opt(OC_block) + \
    Martel.Group("reference_block", Martel.Rep(reference)) + \
    comment + \
    Martel.Opt(DR_block) + \
    Martel.Opt(KW_block) + \
    Martel.Opt(feature_block) + \
    sequence + \
    end
    )

format = Martel.ParseRecords("trembl14", record,
                             RecordReader.EndsWith, ("//\n",) )
