# This is a very basic program to markup SWISS-PROT records into HTML.

# Because it uses the SAX layer directly, it has to do a lot of work
# to accumulate characters over several calls.  This is about the most
# you probably want to do with raw SAX.  The next step would be to
# redo this using DOM since then you could work with the elements
# directly.

import sys, cgi, string
from xml.sax import saxutils, handler
from Martel import RecordReader
from Martel.formats import swissprot38


# Go ahead and add more cross references here.
# Oh, and you think I actually tested these links?
def get_url_for_PFAM(db, primary, secondary, status):
    return "http://www.sanger.ac.uk:80/srs/srsc?[PFAM-id:%s]" % primary

def get_url_for_PROSITE(db, primary, secondary, status):
    return "http://expasy.hcuge.ch/cgi-bin/get-prosite-entry?" + primary

lookup_table = {
    "PFAM": get_url_for_PFAM,
    "PROSITE": get_url_for_PROSITE,
    }

def get_url_for_DR(db, primary, secondary, status):
    x = lookup_table.get(db)
    if x is None:
        return None
    return x(db, primary, secondary, status)

# Many more links are available
def url_for_swissprot_id(id):
    return "http://expasy.hcuge.ch/cgi-bin/get-sprot-entry?" + id

def url_for_swissprot_ac(ac):
    return "http://expasy.hcuge.ch/cgi-bin/get-sprot-entry?" + ac
    
# The SAX callback object which does the hard work
class SPConverter(handler.ContentHandler):
    def __init__(self):
        self.s = ""
        self.store = 0
        self.outfile = sys.stdout
        self._seq_length_flg = 0
        self.highlight_sequence = 0
        self.counter = 0
        self.store_db_reference = 0
        self.database_identifier = ""
        
    def startElement(self, name, attrs):
        if name == "swissprot38_record":
            self.outfile.write("<pre>\n")
            self._seq_length_flg = 0
        elif name == "sequence_length":
            self.store = 1
        elif name in ("entry_name", "ac_number"):
            self.store = 1
        elif name == "CC_copyright_begin":
            self.outfile.write('<font color="gray">')
        elif name == "feature_block":
            self.features = []
        elif name == "feature":
            self.feature_from = None
            self.feature_to = None
        elif name in ("ft_from", "ft_to"):
            self.store = 1
        elif name == "SQ":
            self.outfile.write('<a name="sequence%d">' % self.counter)
            feats_lookup = {}
            for feature in self.features:
                ft_from, ft_to = map(string.strip, feature)
                if ft_from == '?':
                    if ft_to == '?':
                        continue
                    else:
                        ft_from = ft_to
                elif ft_to == '?':
                    ft_to = ft_from
                if ft_from[:1] in "<>":
                    ft_from = ft_from[1:]
                if ft_to[:1] == "<>":
                    ft_to = ft_to[1:]
                # Convert to Python ranges
                for i in range(int(ft_from)-1, int(ft_to)):
                    feats_lookup[i] = 1
            self.feats_lookup = feats_lookup
            self.sequence_count = 0
        elif name == "sequence":
            self.highlight_sequence = 1
        elif name == "database_reference":
            self.store = 1
            self.store_db_reference = 1
            self.database_reference = ""
            self.database_identifier = ""
            self.primary_identifier = None
            self.secondary_identifier = None
            self.status_identifier = None
        elif name in ("database_identifier", "primary_identifier",
                      "secondary_identifier", "status_identifier"):
            self.store = 1

    def characters(self, content):
        if self.store_db_reference:
            self.database_reference = self.database_reference + content
            if self.store:
                self.s = self.s + content
            return

        if self.highlight_sequence == 1:
            n = self.sequence_count
            prev_check = 0
            write = self.outfile.write
            for c in content:
                if c == " ":
                    write(c)
                else:
                    curr_check = self.feats_lookup.get(n)
                    n = n + 1
                    if curr_check and not prev_check:
                        write('<font color="green">')
                    elif not curr_check and prev_check:
                        write('</font>')
                    write(c)
                    prev_check = curr_check
            if prev_check:
                self.outfile.write('</font>')
            self.sequence_count = n
        elif self.store:
            self.s = self.s + content
        else:
            self.outfile.write(cgi.escape(content))
            
    def endElement(self, name):
        if name == "sequence_length":
            if self._seq_length_flg == 1:
                return
            self._seq_length_flg = 1
            self.outfile.write('<a href="#sequence%d">' % self.counter)
        elif name == "entry_name":
            self.outfile.write('<a href="%s">' % url_for_swissprot_id(self.s))
        elif name == "ac_number":
            self.outfile.write('<a href="%s">' % url_for_swissprot_ac(self.s))
        elif name == "ft_from":
            self.feature_from = self.s
        elif name == "ft_to":
            self.feature_to = self.s
        elif name == "database_identifier" or \
             name == "primary_identifier" or \
             name == "secondary_identifier" or \
             name == "status_identifier":
            if name == "database_identifier":
                self.database_identifier = self.s
            elif name == "primary_identifier":
                self.primary_identifier = self.s
            elif name == "secondary_identifier":
                self.secondary_identifier = self.s
            elif name == "status_identifier":
                self.status_identifier = self.s
            self.s = ""
            self.store = 0
        elif name == "feature":
            self.features.append( (self.feature_from, self.feature_to) )
        elif name == "sequence":
            self.highlight_sequence = 0
        elif name == "database_reference":
            self.store_db_reference = 0
            url = get_url_for_DR(self.database_identifier,
                                 self.primary_identifier,
                                 self.secondary_identifier,
                                 self.status_identifier)
            if url is None:
                self.outfile.write(cgi.escape(self.database_reference))
            else:
                self.outfile.write('<a href="%s">%s</a>' % \
                                   (url, cgi.escape(self.database_reference)))
        
        if self.store:
            self.store = 0
            self.outfile.write(cgi.escape(self.s))
            self.s = ""
        
        if name == "swissprot38_record":
            self.outfile.write("</pre>\n")
            self.counter = self.counter + 1
        elif name in ("entry_name", "ac_number", "sequence_length"):
            self.outfile.write("</a>")
        elif name == "CC_copyright_end":
            self.outfile.write('</font>')
        elif name == "SQ_data_block":
            self.highlight_sequence = 0
        elif name == "END":
            self.outfile.write('<hr width="50%">\n')

def main(filename = None):
    if filename is None:
        infile = sys.stdin
    else:
        infile = open(filename)
    parser = swissprot38.format.make_parser()
    parser.setContentHandler(SPConverter())
    parser.setErrorHandler(handler.ErrorHandler())

    parser.parseFile(infile)

if __name__ == "__main__":
    assert len(sys.argv) == 1, "no command line options supported"
    main()
