#
def usage(outfile):
    outfile.write("""\
Usage: toxml [--format <name>]
Convert a file (on stdin) into XML tags (on stdout) using the Martel
format. definition.  If not specified, this will auto-guess the format
""")

import sys, getopt, string
import os
try:
    import cPickle as pickle
except ImportError:
    import pickle
from xml.sax import saxutils, handler
import Martel

cachefilename = "toxml.cache"

def get_parser():
    # see if there's a cache
    parser = None
    try:
        parser = pickle.load(open(cachefilename, "r"))
    except KeyboardInterrupt:
        raise
    except:
        pass

    if parser is not None:
        return parser
    
    # Should dig up the format directory using Martel.__file__
    # then try to read in the available formats.  Next version :)
    from Martel.formats import \
         swissprot38, PDB_2_1, MDL_10_1996, blastall_2_0_10
    format = swissprot38.format | PDB_2_1.format | MDL_10_1996.format | \
             blastall_2_0_10.format

    parser = format.make_parser()

    try:
        outfile = open(cachefilename, "w")
    except IOError:
        return parser
    
    pickle.dump(parser, outfile)
    outfile.close()

    return parser

def main(argv):
    args, files = getopt.getopt(argv[1:], "",
                                ["format=", "help", "version", "clear-cache"])
    assert not files, "files may not be specified"

    format = None
    clear_cache = 0
    for name, val in args:
        if name == "--format":
            format = val
        elif name == "--help":
            usage(sys.stdout)
            sys.exit(0)
        elif name == "--version":
            print "toxml 0.1 - convert a given file into XML"
            sys.exit(0)
        elif name == "--clear-cache":
            clear_cache = 1
        else:
            raise AssertionError("unknown option: %s" % repr(name))

    if clear_cache:
        import os
        try:
            os.unlink(cachefilename)
        except os.error:
            pass

    if format is not None:
        module = __import__("Martel.formats." + format).formats
        module = getattr(module, format)
        format = module.format
        parser = format.make_parser()
    else:
        parser = get_parser()

    parser.setErrorHandler(handler.ErrorHandler())
    parser.setContentHandler(saxutils.XMLGenerator())
    s = sys.stdin.read(10000000)
    if len(s) >= 10000000:
        raise AssertionError, "you really don't want me to parse a 10MB file"
    parser.parseString(s)
    
if __name__ == "__main__":
    main(sys.argv)

